@@ -903,6 +903,12 @@ function update_tempest_conf_file {
     if [[ "$KURYR_CONTROLLER_HA" == "True" ]]; then
         iniset $TEMPEST_CONFIG kuryr_kubernetes ap_ha True
     fi
+    if [[ "$KURYR_K8S_MULTI_WORKER_TESTS" == "True" ]]; then
+        iniset $TEMPEST_CONFIG kuryr_kubernetes multi_worker_setup True
+    fi
+    if [[ "$KURYR_K8S_CLOUD_PROVIDER" == "True" ]]; then
+        iniset $TEMPEST_CONFIG kuryr_kubernetes cloud_provider True
+    fi
 }
 
 source $DEST/kuryr-kubernetes/devstack/lib/kuryr_kubernetes
@@ -1032,9 +1038,9 @@ if [[ "$1" == "stack" && "$2" == "extra" ]]; then
     if is_service_enabled kuryr-kubernetes || [[ ${KURYR_FORCE_IMAGE_BUILD} == "True" ]]; then
         if [ "$KURYR_K8S_CONTAINERIZED_DEPLOYMENT" == "True" ]; then
             if is_service_enabled kuryr-daemon; then
-                build_kuryr_containers $CNI_BIN_DIR $CNI_CONF_DIR True
+                build_kuryr_containers True
             else
-                build_kuryr_containers $CNI_BIN_DIR $CNI_CONF_DIR False
+                build_kuryr_containers False
             fi
         fi
     fi
