@@ -13,6 +13,9 @@
 #    License for the specific language governing permissions and limitations
 #    under the License.
 
+import abc
+import six
+
 import os_vif
 import pyroute2
 from stevedore import driver as stv_driver
@@ -22,6 +25,19 @@ from kuryr_kubernetes import utils
 _BINDING_NAMESPACE = 'kuryr_kubernetes.cni.binding'
 
 
+@six.add_metaclass(abc.ABCMeta)
+class BaseBindingDriver(object):
+    """Interface to attach ports to pods."""
+
+    @abc.abstractmethod
+    def connect(self, vif, ifname, netns, container_id):
+        raise NotImplementedError()
+
+    @abc.abstractmethod
+    def disconnect(self, vif, ifname, netns, container_id):
+        raise NotImplementedError()
+
+
 def _get_binding_driver(vif):
     mgr = stv_driver.DriverManager(namespace=_BINDING_NAMESPACE,
                                    name=type(vif).__name__,
@@ -76,19 +92,19 @@ def _configure_l3(vif, ifname, netns, is_default_gateway):
 
 
 def connect(vif, instance_info, ifname, netns=None, report_health=None,
-            is_default_gateway=True):
+            is_default_gateway=True, container_id=None):
     driver = _get_binding_driver(vif)
     if report_health:
-        report_health(driver.is_healthy())
+        report_health(driver.is_alive())
     os_vif.plug(vif, instance_info)
-    driver.connect(vif, ifname, netns)
+    driver.connect(vif, ifname, netns, container_id)
     _configure_l3(vif, ifname, netns, is_default_gateway)
 
 
 def disconnect(vif, instance_info, ifname, netns=None, report_health=None,
-               **kwargs):
+               container_id=None, **kwargs):
     driver = _get_binding_driver(vif)
     if report_health:
-        report_health(driver.is_healthy())
-    driver.disconnect(vif, ifname, netns)
+        report_health(driver.is_alive())
+    driver.disconnect(vif, ifname, netns, container_id)
     os_vif.unplug(vif, instance_info)
