@@ -33,7 +33,11 @@ kuryr_k8s_opts = [
 daemon_opts = [
     cfg.BoolOpt('daemon_enabled',
                 help=_('Enable CNI Daemon configuration.'),
-                default=False),
+                default=True,
+                deprecated_for_removal=True,
+                deprecated_reason="Deployment without kuryr-daemon is now "
+                                  "deprecated.",
+                deprecated_since="Rocky"),
     cfg.StrOpt('bind_address',
                help=_('Bind address for CNI daemon HTTP server. It is '
                       'recommened to allow only local connections.'),
@@ -58,7 +62,7 @@ daemon_opts = [
                 help=_('Set to True when you are running kuryr-daemon inside '
                        'a Docker container on Kubernetes host. E.g. as '
                        'DaemonSet on Kubernetes cluster Kuryr is supposed to '
-                       'provide networking for. This mainly means that'
+                       'provide networking for. This mainly means that '
                        'kuryr-daemon will look for network namespaces in '
                        '$netns_proc_dir instead of /proc.'),
                 default=False),
@@ -70,6 +74,11 @@ daemon_opts = [
                       "host network namespaces, which is essential for Kuryr "
                       "to work."),
                default=None),
+    cfg.IntOpt('cni_failures_count',
+               help=_('Maximum number of consecutive failures of kuryr-daemon '
+                      'when processing requests. If this number is exceeded, '
+                      'kuryr-daemon will be marked as unhealthy.'),
+               default=3),
 ]
 
 k8s_opts = [
@@ -99,6 +108,14 @@ k8s_opts = [
                help=_("The driver to determine OpenStack "
                       "project for services"),
                default='default'),
+    cfg.StrOpt('namespace_project_driver',
+               help=_("The driver to determine OpenStack "
+                      "project for namespaces"),
+               default='default'),
+    cfg.StrOpt('network_policy_project_driver',
+               help=_("The driver to determine OpenStack "
+                      "project for network policies"),
+               default='default'),
     cfg.StrOpt('pod_subnets_driver',
                help=_("The driver to determine Neutron "
                       "subnets for pod ports"),
@@ -121,7 +138,15 @@ k8s_opts = [
     cfg.StrOpt('endpoints_lbaas_driver',
                help=_("The driver that provides LoadBalancers for "
                       "Kubernetes Endpoints"),
-               default='lbaasv2'),
+               default='lbaasv2',
+               deprecated_for_removal=True,
+               deprecated_reason="Default and supported lbaas endpoint is "
+                                 "now Octavia",
+               deprecated_since="Rocky"),
+    cfg.StrOpt('endpoints_driver_octavia_provider',
+               help=_("The Octavia load balancer provider that will be used "
+                      "to support Kubernetes Endpoints"),
+               default='default'),
     cfg.StrOpt('vif_pool_driver',
                help=_("The driver that manages VIFs pools for "
                       "Kubernetes Pods"),
@@ -140,6 +165,25 @@ k8s_opts = [
     cfg.IntOpt('watch_retry_timeout',
                help=_('Time (in seconds) the watcher retries watching for.'),
                default=60),
+    cfg.ListOpt('enabled_handlers',
+                help=_("The comma-separated handlers that should be "
+                       "registered for watching in the pipeline."),
+                default=['vif', 'lb', 'lbaasspec']),
+    cfg.BoolOpt('controller_ha',
+                help=_('Enable kuryr-controller active/passive HA. Only '
+                       'supported in containerized deployments on Kubernetes '
+                       'or OpenShift.'),
+                default=False),
+    cfg.PortOpt('controller_ha_elector_port',
+                help=_('Port on which leader-elector pod is listening to.'),
+                default=16401),
+    cfg.StrOpt('network_policy_driver',
+               help=_("Driver for network policies"),
+               default='default'),
+    cfg.ListOpt('multi_vif_drivers',
+                help=_("The drivers that provide additional VIFs for "
+                       "Kubernetes Pods."),
+                default='noop'),
 ]
 
 neutron_defaults = [
@@ -161,6 +205,9 @@ neutron_defaults = [
     cfg.StrOpt('external_svc_subnet',
                help=_("Optional external subnet ID for Kubernetes services"),
                default=None),
+    cfg.IntOpt('network_device_mtu',
+               help='Default MTU setting for network interface.',
+               default=1500,),
     cfg.IntOpt('lbaas_activation_timeout',
                help=_("Time (in seconds) that kuryr controller waits for "
                       "neutron LBaaS to be activated"),
@@ -172,6 +219,11 @@ octavia_defaults = [
                help=_("Define the communication mode between load balanacer "
                       "and its members"),
                default='L3'),
+    cfg.StrOpt('sg_mode',
+               help=_("Define the LBaaS SG policy."),
+               choices=[('create', 'replace the VIP SG with a new one'),
+                        ('update', 'add rules to the existing VIP SG')],
+               default='update'),
 ]
 
 cache_defaults = [
@@ -183,6 +235,33 @@ cache_defaults = [
                default="dogpile.cache.memory"),
 ]
 
+ingress = [
+    cfg.StrOpt('l7_router_uuid',
+               help=_("UUID of the L7 Router")),
+]
+
+nested_vif_driver_opts = [
+    cfg.StrOpt('worker_nodes_subnet',
+               help=_("Neutron subnet ID for k8s worker node vms."),
+               default=''),
+]
+
+DEFAULT_PHYSNET_SUBNET_MAPPINGS = {}
+DEFAULT_DEVICE_MAPPINGS = []
+sriov_opts = [
+    cfg.DictOpt('default_physnet_subnets',
+                help=_("A mapping of default subnets for certain physnets "
+                       "in a form of physnet-name:<SUBNET-ID>"),
+                default=DEFAULT_PHYSNET_SUBNET_MAPPINGS),
+    cfg.ListOpt('physical_device_mappings',
+                default=DEFAULT_DEVICE_MAPPINGS,
+                help=_("Comma-separated list of "
+                       "<physical_network>:<network_device> tuples mapping "
+                       "physical network names to the agent's node-specific "
+                       "physical network device interfaces of SR-IOV physical "
+                       "function to be used for VLAN networks.")),
+]
+
 
 CONF = cfg.CONF
 CONF.register_opts(kuryr_k8s_opts)
@@ -191,6 +270,9 @@ CONF.register_opts(k8s_opts, group='kubernetes')
 CONF.register_opts(neutron_defaults, group='neutron_defaults')
 CONF.register_opts(octavia_defaults, group='octavia_defaults')
 CONF.register_opts(cache_defaults, group='cache_defaults')
+CONF.register_opts(ingress, group='ingress')
+CONF.register_opts(nested_vif_driver_opts, group='pod_vif_nested')
+CONF.register_opts(sriov_opts, group='sriov')
 
 CONF.register_opts(lib_config.core_opts)
 CONF.register_opts(lib_config.binding_opts, 'binding')
