@@ -247,6 +247,48 @@ class PodSecurityGroupsDriver(DriverBase):
         """
         raise NotImplementedError()
 
+    def create_sg_rules(self, pod):
+        """Create security group rules for a pod.
+
+        :param pod: dict containing Kubernetes Pod object
+        """
+        raise NotImplementedError()
+
+    def delete_sg_rules(self, pod):
+        """Delete security group rules for a pod
+
+        :param pod: dict containing Kubernetes Pod object
+        """
+        raise NotImplementedError()
+
+    def update_sg_rules(self, pod):
+        """Update security group rules for a pod
+
+        :param pod: dict containing Kubernetes Pod object
+        """
+        raise NotImplementedError()
+
+    def delete_namespace_sg_rules(self, namespace):
+        """Delete security group rule associated to a namespace.
+
+        :param namespace: dict containing K8S Namespace object
+        """
+        raise NotImplementedError()
+
+    def create_namespace_sg_rules(self, namespace):
+        """Create security group rule associated to a namespace.
+
+        :param namespace: dict containing K8S Namespace object
+        """
+        raise NotImplementedError()
+
+    def update_namespace_sg_rules(self, namespace):
+        """Update security group rule associated to a namespace.
+
+        :param namespace: dict containing K8S Namespace object
+        """
+        raise NotImplementedError()
+
 
 @six.add_metaclass(abc.ABCMeta)
 class ServiceSecurityGroupsDriver(DriverBase):
@@ -757,7 +799,7 @@ class NetworkPolicyDriver(DriverBase):
         :param policy: dict containing Kubernetes NP object
         :param selector: (optional) specifc pod selector
         :returns: list of Pods objects affected by the policy or the selector
-                  if it is pased
+                  if it is passed
         """
         raise NotImplementedError()
 
