@@ -31,17 +31,12 @@ from kuryr_kubernetes import constants as const
 from kuryr_kubernetes.controller.drivers import base
 from kuryr_kubernetes import exceptions as k_exc
 from kuryr_kubernetes.objects import lbaas as obj_lbaas
+from kuryr_kubernetes import utils
 
 CONF = cfg.CONF
 LOG = logging.getLogger(__name__)
 
 _ACTIVATION_TIMEOUT = CONF.neutron_defaults.lbaas_activation_timeout
-
-_PROVIDER_SUPPORTED_LISTENER_PROT = {
-    'amphora': ['HTTP', 'HTTPS', 'TCP', 'UDP'],
-    'ovn': ['TCP', 'UDP'],
-    'haproxy':  ['HTTP', 'HTTPS', 'TCP']}
-
 _L7_POLICY_ACT_REDIRECT_TO_POOL = 'REDIRECT_TO_POOL'
 # NOTE(yboaron):Prior to sending create request to Octavia, LBaaS driver
 # verifies that LB is in a stable state by polling LB's provisioning_status
@@ -170,6 +165,106 @@ class LBaaSv2Driver(base.LBaaSDriver):
                     LOG.exception('Failed when creating security group rule '
                                   'for listener %s.', listener.name)
 
+    def _create_lb_security_group_rule(self, loadbalancer, listener):
+        neutron = clients.get_neutron_client()
+        sg_id = self._find_listeners_sg(loadbalancer)
+        # if an SG for the loadbalancer has not being created, create one
+        if not sg_id:
+            sg = neutron.create_security_group({
+                'security_group': {
+                    'name': loadbalancer.name,
+                    'project_id': loadbalancer.project_id,
+                    },
+                })
+            sg_id = sg['security_group']['id']
+            loadbalancer.security_groups.append(sg_id)
+            vip_port = self._get_vip_port(loadbalancer)
+            neutron.update_port(
+                vip_port.get('id'),
+                {'port': {
+                    'security_groups': loadbalancer.security_groups}})
+
+        try:
+            neutron.create_security_group_rule({
+                'security_group_rule': {
+                    'direction': 'ingress',
+                    'port_range_min': listener.port,
+                    'port_range_max': listener.port,
+                    'protocol': listener.protocol,
+                    'security_group_id': sg_id,
+                    'description': listener.name,
+                },
+            })
+        except n_exc.NeutronClientException as ex:
+            if ex.status_code != requests.codes.conflict:
+                LOG.exception('Failed when creating security group rule '
+                              'for listener %s.', listener.name)
+
+    def _get_matched_sg_rule(self, rule, lbaas_sg_rules):
+        for lbaas_sg_rule in lbaas_sg_rules:
+            if lbaas_sg_rule['remote_ip_prefix'] == rule['remote_ip_prefix']:
+                return lbaas_sg_rule
+        return None
+
+    def _delete_sg_rule(self, rule, lbaas_sg_rules):
+        neutron = clients.get_neutron_client()
+        sg_rule = self._get_matched_sg_rule(rule, lbaas_sg_rules)
+        if sg_rule:
+            LOG.debug("Deleting sg rule: %r", sg_rule['id'])
+            neutron.delete_security_group_rule(sg_rule['id'])
+
+    def _apply_members_security_groups(self, loadbalancer, port, target_port,
+                                       protocol, sg_rule_name):
+        neutron = clients.get_neutron_client()
+        if CONF.octavia_defaults.sg_mode == 'create':
+            sg_id = self._find_listeners_sg(loadbalancer)
+        else:
+            sg_id = self._get_vip_port(loadbalancer).get('security_groups')[0]
+
+        lbaas_sg_rules = neutron.list_security_group_rules(
+            security_group_id=sg_id)
+
+        # Check if Network Policy allows listener on the pods
+        for sg in loadbalancer.security_groups:
+            if sg != sg_id:
+                rules = neutron.list_security_group_rules(
+                    security_group_id=sg)
+                for rule in rules['security_group_rules']:
+                    # copying ingress rules with same protocol onto the
+                    # loadbalancer sg rules
+                    # NOTE(ltomasbo): NP security groups only have
+                    # remote_ip_prefix, not remote_group_id, therefore only
+                    # applying the ones with remote_ip_prefix
+                    if (rule['protocol'] == protocol.lower() and
+                            rule['direction'] == 'ingress' and
+                            rule['remote_ip_prefix']):
+                        # If listener port not in allowed range, skip
+                        min_port = rule.get('port_range_min')
+                        max_port = rule.get('port_range_max')
+                        if (min_port and target_port not in range(min_port,
+                                                                  max_port+1)):
+                            self._delete_sg_rule(
+                                rule, lbaas_sg_rules['security_group_rules'])
+                            continue
+                        try:
+                            neutron.create_security_group_rule({
+                                'security_group_rule': {
+                                    'direction': 'ingress',
+                                    'port_range_min': port,
+                                    'port_range_max': port,
+                                    'protocol': protocol,
+                                    'remote_ip_prefix': rule[
+                                        'remote_ip_prefix'],
+                                    'security_group_id': sg_id,
+                                    'description': sg_rule_name,
+                                },
+                            })
+                        except n_exc.NeutronClientException as ex:
+                            if ex.status_code != requests.codes.conflict:
+                                LOG.exception('Failed when creating security '
+                                              'group rule for listener %s.',
+                                              sg_rule_name)
+
     def _extend_lb_security_group_rules(self, loadbalancer, listener):
         neutron = clients.get_neutron_client()
 
@@ -213,7 +308,7 @@ class LBaaSv2Driver(base.LBaaSDriver):
                                       'rule for listener %s.', listener.name)
 
         # ensure routes have access to the services
-        service_subnet_cidr = self._get_subnet_cidr(loadbalancer.subnet_id)
+        service_subnet_cidr = utils.get_subnet_cidr(loadbalancer.subnet_id)
         try:
             # add access from service subnet
             neutron.create_security_group_rule({
@@ -232,7 +327,7 @@ class LBaaSv2Driver(base.LBaaSDriver):
             # support
             worker_subnet_id = CONF.pod_vif_nested.worker_nodes_subnet
             if worker_subnet_id:
-                worker_subnet_cidr = self._get_subnet_cidr(worker_subnet_id)
+                worker_subnet_cidr = utils.get_subnet_cidr(worker_subnet_id)
                 neutron.create_security_group_rule({
                     'security_group_rule': {
                         'direction': 'ingress',
@@ -252,32 +347,20 @@ class LBaaSv2Driver(base.LBaaSDriver):
 
     def _ensure_security_group_rules(self, loadbalancer, listener,
                                      service_type):
+        namespace_isolation = (
+            'namespace' in CONF.kubernetes.enabled_handlers and
+            CONF.kubernetes.service_security_groups_driver == 'namespace')
+        create_sg = CONF.octavia_defaults.sg_mode == 'create'
+
         if loadbalancer.provider == const.NEUTRON_LBAAS_HAPROXY_PROVIDER:
             self._ensure_lb_security_group_rule(loadbalancer, listener)
-        elif service_type == 'ClusterIP':
+        elif namespace_isolation and service_type == 'ClusterIP':
             self._extend_lb_security_group_rules(loadbalancer, listener)
+        elif create_sg:
+            self._create_lb_security_group_rule(loadbalancer, listener)
 
     def ensure_listener(self, loadbalancer, protocol, port,
                         service_type='ClusterIP'):
-
-        # NOTE(yboaron): Since retrieving Octavia capabilities/version is not
-        # supported via  the OpenstackSdk, the list of allowed listener's
-        # protocols will be defined statically.
-        # Kuryr still need to handle the case in which listener's protocol
-        # (e.g: UDP) is not supported by Octavia.
-        provider = loadbalancer.provider or 'amphora'
-        try:
-            if protocol not in _PROVIDER_SUPPORTED_LISTENER_PROT[provider]:
-                LOG.info("Protocol: %(prot)s: is not supported by "
-                         "%(provider)s",
-                         {'prot': protocol, 'provider': provider})
-                return None
-        except KeyError:
-            LOG.info("Provider %(provider)s doesnt exist in "
-                     "_PROVIDER_SUPPORTED_LISTENER_PROT",
-                     {'provider': provider})
-            return None
-
         name = "%s:%s:%s" % (loadbalancer.name, protocol, port)
         listener = obj_lbaas.LBaaSListener(name=name,
                                            project_id=loadbalancer.project_id,
@@ -304,7 +387,10 @@ class LBaaSv2Driver(base.LBaaSDriver):
                       lbaas.delete_listener,
                       listener.id)
 
-        sg_id = self._find_listeners_sg(loadbalancer)
+        if CONF.octavia_defaults.sg_mode == 'create':
+            sg_id = self._find_listeners_sg(loadbalancer)
+        else:
+            sg_id = self._get_vip_port(loadbalancer).get('security_groups')[0]
         if sg_id:
             rules = neutron.list_security_group_rules(
                 security_group_id=sg_id, description=listener.name)
@@ -346,7 +432,7 @@ class LBaaSv2Driver(base.LBaaSDriver):
 
     def ensure_member(self, loadbalancer, pool,
                       subnet_id, ip, port, target_ref_namespace,
-                      target_ref_name):
+                      target_ref_name, listener_port=None):
         name = ("%s/%s" % (target_ref_namespace, target_ref_name))
         name += ":%s" % port
         member = obj_lbaas.LBaaSMember(name=name,
@@ -355,9 +441,19 @@ class LBaaSv2Driver(base.LBaaSDriver):
                                        subnet_id=subnet_id,
                                        ip=ip,
                                        port=port)
-        return self._ensure_provisioned(loadbalancer, member,
-                                        self._create_member,
-                                        self._find_member)
+        result = self._ensure_provisioned(loadbalancer, member,
+                                          self._create_member,
+                                          self._find_member)
+
+        network_policy = (
+            'policy' in CONF.kubernetes.enabled_handlers and
+            CONF.kubernetes.service_security_groups_driver == 'policy')
+        if network_policy and listener_port:
+            protocol = pool.protocol
+            sg_rule_name = pool.name
+            self._apply_members_security_groups(loadbalancer, listener_port,
+                                                port, protocol, sg_rule_name)
+        return result
 
     def release_member(self, loadbalancer, member):
         lbaas = clients.get_loadbalancer_client()
@@ -380,15 +476,6 @@ class LBaaSv2Driver(base.LBaaSDriver):
 
         return None
 
-    def _get_subnet_cidr(self, subnet_id):
-        neutron = clients.get_neutron_client()
-        try:
-            subnet_obj = neutron.show_subnet(subnet_id)
-        except n_exc.NeutronClientException:
-            LOG.exception("Subnet %s CIDR not found!", subnet_id)
-            raise
-        return subnet_obj.get('subnet')['cidr']
-
     def _create_loadbalancer(self, loadbalancer):
         lbaas = clients.get_loadbalancer_client()
 
