@@ -17,10 +17,12 @@ from oslo_log import log as logging
 from neutronclient.common import exceptions as n_exc
 
 from kuryr_kubernetes import clients
+from kuryr_kubernetes import config
 from kuryr_kubernetes import constants
 from kuryr_kubernetes.controller.drivers import base
-from kuryr_kubernetes.controller.drivers import utils
+from kuryr_kubernetes.controller.drivers import utils as driver_utils
 from kuryr_kubernetes import exceptions
+from kuryr_kubernetes import utils
 
 LOG = logging.getLogger(__name__)
 
@@ -93,14 +95,14 @@ class NetworkPolicyDriver(base.NetworkPolicyDriver):
                               current_sg_rules]
         for sg_rule in sg_rules_to_delete:
             try:
-                self._delete_security_group_rule(sgr_ids[sg_rule])
+                driver_utils.delete_security_group_rule(sgr_ids[sg_rule])
             except n_exc.NotFound:
                 LOG.debug('Trying to delete non existing sg_rule %s', sg_rule)
         # Create new rules that weren't already on the security group
         sg_rules_to_add = [rule for rule in current_sg_rules if rule not in
                            existing_sg_rules]
         for sg_rule in sg_rules_to_add:
-            sgr_id = self._create_security_group_rule(sg_rule)
+            sgr_id = driver_utils.create_security_group_rule(sg_rule)
             if sg_rule['security_group_rule'].get('direction') == 'ingress':
                 for i_rule in i_rules:
                     if sg_rule == i_rule:
@@ -111,17 +113,9 @@ class NetworkPolicyDriver(base.NetworkPolicyDriver):
                         e_rule["security_group_rule"]["id"] = sgr_id
         # Annotate kuryrnetpolicy CRD with current policy and ruleset
         pod_selector = policy['spec'].get('podSelector')
-        LOG.debug('Patching KuryrNetPolicy CRD %s' % crd_name)
-        try:
-            self.kubernetes.patch('spec', crd['metadata']['selfLink'],
-                                  {'ingressSgRules': i_rules,
-                                   'egressSgRules': e_rules,
-                                   'podSelector': pod_selector,
-                                   'networkpolicy_spec': policy['spec']})
+        driver_utils.patch_kuryr_crd(crd, i_rules, e_rules, pod_selector,
+                                     np_spec=policy['spec'])
 
-        except exceptions.K8sClientException:
-            LOG.exception('Error updating kuryrnetpolicy CRD %s', crd_name)
-            raise
         if existing_pod_selector != pod_selector:
             return existing_pod_selector
         return False
@@ -150,11 +144,26 @@ class NetworkPolicyDriver(base.NetworkPolicyDriver):
             sg_id = sg['security_group']['id']
             i_rules, e_rules = self.parse_network_policy_rules(policy, sg_id)
             for i_rule in i_rules:
-                sgr_id = self._create_security_group_rule(i_rule)
+                sgr_id = driver_utils.create_security_group_rule(i_rule)
                 i_rule['security_group_rule']['id'] = sgr_id
+
             for e_rule in e_rules:
-                sgr_id = self._create_security_group_rule(e_rule)
+                sgr_id = driver_utils.create_security_group_rule(e_rule)
                 e_rule['security_group_rule']['id'] = sgr_id
+
+            # NOTE(ltomasbo): Add extra SG rule to allow traffic from services
+            # subnet
+            svc_cidr = utils.get_subnet_cidr(
+                config.CONF.neutron_defaults.service_subnet)
+            svc_rule = {
+                u'security_group_rule': {
+                    u'ethertype': 'IPv4',
+                    u'security_group_id': sg_id,
+                    u'direction': 'ingress',
+                    u'description': 'Kuryr-Kubernetes NetPolicy SG rule',
+                    u'remote_ip_prefix': svc_cidr
+                }}
+            driver_utils.create_security_group_rule(svc_rule)
         except (n_exc.NeutronClientException, exceptions.ResourceNotReady):
             LOG.exception("Error creating security group for network policy "
                           " %s", policy['metadata']['name'])
@@ -183,49 +192,38 @@ class NetworkPolicyDriver(base.NetworkPolicyDriver):
     def _get_pods_ips(self, pod_selector, namespace=None,
                       namespace_selector=None):
         ips = []
-        matching_pods = []
+        matching_pods = {"items": []}
         if namespace_selector:
-            matching_namespaces = utils.get_namespaces(namespace_selector)
+            matching_namespaces = driver_utils.get_namespaces(
+                namespace_selector)
             for ns in matching_namespaces.get('items'):
-                matching_pods = utils.get_pods(pod_selector,
-                                               ns['metadata']['name'])
+                matching_pods = driver_utils.get_pods(pod_selector,
+                                                      ns['metadata']['name'])
         else:
-            matching_pods = utils.get_pods(pod_selector, namespace)
+            matching_pods = driver_utils.get_pods(pod_selector, namespace)
         for pod in matching_pods.get('items'):
             if pod['status']['podIP']:
-                ips.append(pod['status']['podIP'])
+                pod_ip = pod['status']['podIP']
+                ns = pod['metadata']['namespace']
+                ips.append({'cidr': pod_ip, 'namespace': ns})
         return ips
 
-    def _get_namespace_subnet_cidr(self, namespace):
-        try:
-            ns_annotations = namespace['metadata']['annotations']
-            ns_name = ns_annotations[constants.K8S_ANNOTATION_NET_CRD]
-        except KeyError:
-            LOG.exception('Namespace handler must be enabled to support '
-                          'Network Policies with namespaceSelector')
-            raise exceptions.ResourceNotReady(namespace)
-        try:
-            net_crd = self.kubernetes.get('{}/kuryrnets/{}'.format(
-                constants.K8S_API_CRD, ns_name))
-        except exceptions.K8sClientException:
-            LOG.exception("Kubernetes Client Exception.")
-            raise
-        return net_crd['spec']['subnetCIDR']
-
     def _get_namespaces_cidr(self, namespace_selector, namespace=None):
         cidrs = []
         if not namespace_selector and namespace:
             ns = self.kubernetes.get(
                 '{}/namespaces/{}'.format(constants.K8S_API_BASE, namespace))
-            ns_cidr = self._get_namespace_subnet_cidr(ns)
-            cidrs.append(ns_cidr)
+            ns_cidr = driver_utils.get_namespace_subnet_cidr(ns)
+            cidrs.append({'cidr': ns_cidr, 'namespace': namespace})
         else:
-            matching_namespaces = utils.get_namespaces(namespace_selector)
+            matching_namespaces = driver_utils.get_namespaces(
+                namespace_selector)
             for ns in matching_namespaces.get('items'):
                 # NOTE(ltomasbo): This requires the namespace handler to be
                 # also enabled
-                ns_cidr = self._get_namespace_subnet_cidr(ns)
-                cidrs.append(ns_cidr)
+                ns_cidr = driver_utils.get_namespace_subnet_cidr(ns)
+                ns_name = ns['metadata']['name']
+                cidrs.append({'cidr': ns_cidr, 'namespace': ns_name})
         return cidrs
 
     def _parse_selectors(self, rule_block, rule_direction, policy_namespace):
@@ -247,12 +245,12 @@ class NetworkPolicyDriver(base.NetworkPolicyDriver):
             elif namespace_selector:
                 selectors = True
                 if pod_selector:
-                    # allow matching pods on maching namespaces
+                    # allow matching pods on matching namespaces
                     allowed_cidrs.extend(self._get_pods_ips(
                         pod_selector,
                         namespace_selector=namespace_selector))
                 else:
-                    # allow from/to all on the maching namespaces
+                    # allow from/to all on the matching namespaces
                     allowed_cidrs.extend(self._get_namespaces_cidr(
                         namespace_selector))
             else:
@@ -270,6 +268,7 @@ class NetworkPolicyDriver(base.NetworkPolicyDriver):
                     allowed_cidrs.extend(self._get_pods_ips(
                         pod_selector,
                         namespace=policy_namespace))
+
         return allow_all, selectors, allowed_cidrs
 
     def _parse_sg_rules(self, sg_rule_body_list, direction, policy, sg_id):
@@ -285,7 +284,7 @@ class NetworkPolicyDriver(base.NetworkPolicyDriver):
         if rule_list[0] == {}:
             LOG.debug('Applying default all open policy from %s',
                       policy['metadata']['selfLink'])
-            rule = self._create_security_group_rule_body(
+            rule = driver_utils.create_security_group_rule_body(
                 sg_id, direction, port_range_min=1, port_range_max=65535)
             sg_rule_body_list.append(rule)
 
@@ -299,31 +298,35 @@ class NetworkPolicyDriver(base.NetworkPolicyDriver):
                 for port in rule_block['ports']:
                     if allowed_cidrs or allow_all or selectors:
                         for cidr in allowed_cidrs:
-                            rule = self._create_security_group_rule_body(
-                                sg_id, direction, port.get('port'),
-                                protocol=port.get('protocol'),
-                                cidr=cidr)
+                            rule = (
+                                driver_utils.create_security_group_rule_body(
+                                    sg_id, direction, port.get('port'),
+                                    protocol=port.get('protocol'),
+                                    cidr=cidr.get('cidr'),
+                                    namespace=cidr.get('namespace')))
                             sg_rule_body_list.append(rule)
                         if allow_all:
-                            rule = self._create_security_group_rule_body(
-                                sg_id, direction, port.get('port'),
-                                protocol=port.get('protocol'))
+                            rule = (
+                                driver_utils.create_security_group_rule_body(
+                                    sg_id, direction, port.get('port'),
+                                    protocol=port.get('protocol')))
                             sg_rule_body_list.append(rule)
                     else:
-                        rule = self._create_security_group_rule_body(
+                        rule = driver_utils.create_security_group_rule_body(
                             sg_id, direction, port.get('port'),
                             protocol=port.get('protocol'))
                         sg_rule_body_list.append(rule)
             elif allowed_cidrs or allow_all or selectors:
                 for cidr in allowed_cidrs:
-                    rule = self._create_security_group_rule_body(
+                    rule = driver_utils.create_security_group_rule_body(
                         sg_id, direction,
                         port_range_min=1,
                         port_range_max=65535,
-                        cidr=cidr)
+                        cidr=cidr.get('cidr'),
+                        namespace=cidr.get('namespace'))
                     sg_rule_body_list.append(rule)
                 if allow_all:
-                    rule = self._create_security_group_rule_body(
+                    rule = driver_utils.create_security_group_rule_body(
                         sg_id, direction,
                         port_range_min=1,
                         port_range_max=65535)
@@ -353,59 +356,6 @@ class NetworkPolicyDriver(base.NetworkPolicyDriver):
 
         return ingress_sg_rule_body_list, egress_sg_rule_body_list
 
-    def _create_security_group_rule_body(
-            self, security_group_id, direction, port_range_min,
-            port_range_max=None, protocol=None, ethertype='IPv4', cidr=None,
-            description="Kuryr-Kubernetes NetPolicy SG rule"):
-        if not port_range_min:
-            port_range_min = 1
-            port_range_max = 65535
-        elif not port_range_max:
-            port_range_max = port_range_min
-        if not protocol:
-            protocol = 'TCP'
-        security_group_rule_body = {
-            u'security_group_rule': {
-                u'ethertype': ethertype,
-                u'security_group_id': security_group_id,
-                u'description': description,
-                u'direction': direction,
-                u'protocol': protocol.lower(),
-                u'port_range_min': port_range_min,
-                u'port_range_max': port_range_max
-            }
-        }
-        if cidr:
-            security_group_rule_body[u'security_group_rule'][
-                u'remote_ip_prefix'] = cidr
-        LOG.debug("Creating sg rule body %s", security_group_rule_body)
-        return security_group_rule_body
-
-    def _create_security_group_rule(self, body):
-        sgr = ''
-        try:
-            sgr = self.neutron.create_security_group_rule(
-                body=body)
-        except n_exc.Conflict:
-            LOG.debug("Failed to create already existing security group "
-                      "rule %s", body)
-        except n_exc.NeutronClientException:
-            LOG.debug("Error creating security group rule")
-            raise
-        return sgr["security_group_rule"]["id"]
-
-    def _delete_security_group_rule(self, security_group_rule_id):
-        try:
-            self.neutron.delete_security_group_rule(
-                security_group_rule=security_group_rule_id)
-        except n_exc.NotFound:
-            LOG.debug("Error deleting security group rule as it does not "
-                      "exist: %s", security_group_rule_id)
-        except n_exc.NeutronClientException:
-            LOG.debug("Error deleting security group rule: %s",
-                      security_group_rule_id)
-            raise
-
     def release_network_policy(self, netpolicy_crd):
         if netpolicy_crd is not None:
             try:
@@ -460,7 +410,6 @@ class NetworkPolicyDriver(base.NetworkPolicyDriver):
         netpolicy_crd_name = "np-" + networkpolicy_name
         namespace = policy['metadata']['namespace']
         pod_selector = policy['spec'].get('podSelector')
-
         netpolicy_crd = {
             'apiVersion': 'openstack.org/v1',
             'kind': constants.K8S_OBJ_KURYRNETPOLICY,
@@ -515,7 +464,7 @@ class NetworkPolicyDriver(base.NetworkPolicyDriver):
             pod_selector = policy['spec'].get('podSelector')
         if pod_selector:
             policy_namespace = policy['metadata']['namespace']
-            pods = utils.get_pods(pod_selector, policy_namespace)
+            pods = driver_utils.get_pods(pod_selector, policy_namespace)
             return pods.get('items')
         else:
             # NOTE(ltomasbo): It affects all the pods on the namespace
