@@ -76,6 +76,8 @@ class K8sClient(object):
         response = requests.get(url, cert=self.cert,
                                 verify=self.verify_server,
                                 headers=header)
+        if response.status_code == requests.codes.not_found:
+            raise exc.K8sResourceNotFound(response.text)
         if not response.ok:
             raise exc.K8sClientException(response.text)
         result = response.json() if json else response.text
@@ -90,14 +92,13 @@ class K8sClient(object):
 
         return url, header
 
-    def patch_status(self, path, data):
-
-        LOG.debug("Patch_status %(path)s: %(data)s", {
+    def patch(self, field, path, data):
+        LOG.debug("Patch %(path)s: %(data)s", {
             'path': path, 'data': data})
-        path = path + '/status'
+        if field == 'status':
+            path = path + '/' + str(field)
         url, header = self._get_url_and_header(path)
-
-        response = requests.patch(url, json={"status": data},
+        response = requests.patch(url, json={field: data},
                                   headers=header, cert=self.cert,
                                   verify=self.verify_server)
         if response.ok:
@@ -128,10 +129,7 @@ class K8sClient(object):
                                    verify=self.verify_server, headers=header)
         if response.ok:
             return response.json()
-        else:
-            if response.status_code == requests.codes.not_found:
-                raise exc.K8sResourceNotFound(response.text)
-            raise exc.K8sClientException(response)
+        raise exc.K8sClientException(response)
 
     def annotate(self, path, annotations, resource_version=None):
         """Pushes a resource annotation to the K8s API resource
@@ -198,7 +196,7 @@ class K8sClient(object):
                                  headers=header)) as response:
                 if not response.ok:
                     raise exc.K8sClientException(response.text)
-                for line in response.iter_lines(delimiter='\n'):
-                    line = line.strip()
+                for line in response.iter_lines():
+                    line = line.decode('utf-8').strip()
                     if line:
                         yield jsonutils.loads(line)
