@@ -19,6 +19,7 @@ from kuryr_kubernetes.controller.drivers import network_policy
 from kuryr_kubernetes import exceptions
 from kuryr_kubernetes.tests import base as test_base
 from kuryr_kubernetes.tests.unit import kuryr_fixtures as k_fix
+from kuryr_kubernetes import utils
 
 from neutronclient.common import exceptions as n_exc
 
@@ -119,6 +120,7 @@ class TestNetworkPolicyDriver(test_base.TestCase):
                       'security_group_id': self._sg_id,
                       'id': mock.sentinel.id
                       }}],
+                'networkpolicy_spec': self._policy['spec'],
                 'securityGroupId': self._sg_id,
                 'securityGroupName': mock.sentinel.sg_name}}
 
@@ -184,11 +186,15 @@ class TestNetworkPolicyDriver(test_base.TestCase):
                        '_add_kuryrnetpolicy_crd')
     @mock.patch.object(network_policy.NetworkPolicyDriver,
                        'parse_network_policy_rules')
-    def test_create_security_group_rules_from_network_policy(self, m_parse,
+    @mock.patch.object(utils, 'get_subnet_cidr')
+    def test_create_security_group_rules_from_network_policy(self, m_utils,
+                                                             m_parse,
                                                              m_add_crd,
                                                              m_get_crd):
         self._driver.neutron.create_security_group.return_value = {
             'security_group': {'id': mock.sentinel.id}}
+        m_utils.get_subnet_cidr.return_value = {
+            'subnet': {'cidr': mock.sentinel.cidr}}
         m_parse.return_value = (self._i_rules, self._e_rules)
         self._driver.neutron.create_security_group_rule.return_value = {
             'security_group_rule': {'id': mock.sentinel.id}}
@@ -203,10 +209,13 @@ class TestNetworkPolicyDriver(test_base.TestCase):
                        '_add_kuryrnetpolicy_crd')
     @mock.patch.object(network_policy.NetworkPolicyDriver,
                        'parse_network_policy_rules')
-    def test_create_security_group_rules_with_k8s_exc(self, m_parse,
+    @mock.patch.object(utils, 'get_subnet_cidr')
+    def test_create_security_group_rules_with_k8s_exc(self, m_utils, m_parse,
                                                       m_add_crd, m_get_crd):
         self._driver.neutron.create_security_group.return_value = {
             'security_group': {'id': mock.sentinel.id}}
+        m_utils.get_subnet_cidr.return_value = {
+            'subnet': {'cidr': mock.sentinel.cidr}}
         m_parse.return_value = (self._i_rules, self._e_rules)
         m_get_crd.side_effect = exceptions.K8sClientException
         self._driver.neutron.create_security_group_rule.return_value = {
@@ -223,10 +232,13 @@ class TestNetworkPolicyDriver(test_base.TestCase):
                        '_add_kuryrnetpolicy_crd')
     @mock.patch.object(network_policy.NetworkPolicyDriver,
                        'parse_network_policy_rules')
-    def test_create_security_group_rules_error_add_crd(self, m_parse,
+    @mock.patch.object(utils, 'get_subnet_cidr')
+    def test_create_security_group_rules_error_add_crd(self, m_utils, m_parse,
                                                        m_add_crd, m_get_crd):
         self._driver.neutron.create_security_group.return_value = {
             'security_group': {'id': mock.sentinel.id}}
+        m_utils.get_subnet_cidr.return_value = {
+            'subnet': {'cidr': mock.sentinel.cidr}}
         m_parse.return_value = (self._i_rules, self._e_rules)
         m_add_crd.side_effect = exceptions.K8sClientException
         self._driver.neutron.create_security_group_rule.return_value = {
@@ -245,8 +257,8 @@ class TestNetworkPolicyDriver(test_base.TestCase):
             self._driver.create_security_group_rules_from_network_policy,
             self._policy, self._project_id)
 
-    @mock.patch.object(network_policy.NetworkPolicyDriver,
-                       '_create_security_group_rule')
+    @mock.patch('kuryr_kubernetes.controller.drivers.utils.'
+                'create_security_group_rule')
     @mock.patch.object(network_policy.NetworkPolicyDriver,
                        'get_kuryrnetpolicy_crd')
     @mock.patch.object(network_policy.NetworkPolicyDriver,
@@ -261,8 +273,8 @@ class TestNetworkPolicyDriver(test_base.TestCase):
             policy)
         m_parse.assert_called_with(policy, self._sg_id)
 
-    @mock.patch.object(network_policy.NetworkPolicyDriver,
-                       '_create_security_group_rule')
+    @mock.patch('kuryr_kubernetes.controller.drivers.utils.'
+                'create_security_group_rule')
     @mock.patch.object(network_policy.NetworkPolicyDriver,
                        'get_kuryrnetpolicy_crd')
     @mock.patch.object(network_policy.NetworkPolicyDriver,
@@ -290,7 +302,7 @@ class TestNetworkPolicyDriver(test_base.TestCase):
         self.kubernetes.get.side_effect = [{'items': [pod]}, net_crd]
 
         resp = self._driver._get_namespaces_cidr(namespace_selector)
-        self.assertEqual([subnet_cidr], resp)
+        self.assertEqual(subnet_cidr, resp[0].get('cidr'))
         self.kubernetes.get.assert_called()
 
     def test_get_namespaces_cidr_no_matches(self):
@@ -313,20 +325,20 @@ class TestNetworkPolicyDriver(test_base.TestCase):
 
     @mock.patch.object(network_policy.NetworkPolicyDriver,
                        '_get_namespaces_cidr')
-    @mock.patch.object(network_policy.NetworkPolicyDriver,
-                       '_create_security_group_rule_body')
+    @mock.patch('kuryr_kubernetes.controller.drivers.utils.'
+                'create_security_group_rule_body')
     def test_parse_network_policy_rules_with_rules(self, m_create,
                                                    m_get_ns_cidr):
         subnet_cidr = '10.10.0.0/24'
-        m_get_ns_cidr.return_value = [subnet_cidr]
+        m_get_ns_cidr.return_value = [{'cidr': subnet_cidr, 'namespace': ''}]
         self._driver.parse_network_policy_rules(self._policy, self._sg_id)
         m_create.assert_called()
         m_get_ns_cidr.assert_called()
 
     @mock.patch.object(network_policy.NetworkPolicyDriver,
                        '_get_namespaces_cidr')
-    @mock.patch.object(network_policy.NetworkPolicyDriver,
-                       '_create_security_group_rule_body')
+    @mock.patch('kuryr_kubernetes.controller.drivers.utils.'
+                'create_security_group_rule_body')
     def test_parse_network_policy_rules_with_no_rules(self, m_create,
                                                       m_get_ns_cidr):
         policy = self._policy.copy()
@@ -342,8 +354,8 @@ class TestNetworkPolicyDriver(test_base.TestCase):
 
     @mock.patch.object(network_policy.NetworkPolicyDriver,
                        '_get_namespaces_cidr')
-    @mock.patch.object(network_policy.NetworkPolicyDriver,
-                       '_create_security_group_rule_body')
+    @mock.patch('kuryr_kubernetes.controller.drivers.utils.'
+                'create_security_group_rule_body')
     def test_parse_network_policy_rules_with_no_pod_selector(self, m_create,
                                                              m_get_ns_cidr):
         policy = self._policy.copy()
@@ -357,29 +369,31 @@ class TestNetworkPolicyDriver(test_base.TestCase):
 
     @mock.patch.object(network_policy.NetworkPolicyDriver,
                        '_get_namespaces_cidr')
-    @mock.patch.object(network_policy.NetworkPolicyDriver,
-                       '_create_security_group_rule_body')
+    @mock.patch('kuryr_kubernetes.controller.drivers.utils.'
+                'create_security_group_rule_body')
     def test_parse_network_policy_rules_with_no_ports(self, m_create,
                                                       m_get_ns_cidr):
         subnet_cidr = '10.10.0.0/24'
-        m_get_ns_cidr.return_value = [subnet_cidr]
+        m_get_ns_cidr.return_value = [{'cidr': subnet_cidr, 'namespace': ''}]
         policy = self._policy.copy()
+        selectors = {'namespaceSelector': {
+                     'matchLabels': {
+                         'project': 'myproject'}}}
         policy['spec']['egress'] = [
             {'to':
-                [{'namespaceSelector': {
-                    'matchLabels': {
-                        'project': 'myproject'}}}]}]
+                [selectors]}]
         policy['spec']['ingress'] = [
             {'from':
-                [{'namespaceSelector': {
-                    'matchLabels': {
-                        'project': 'myproject'}}}]}]
+                [selectors]}]
+        selectors = {'namespace_selector': selectors['namespaceSelector']}
         self._driver.parse_network_policy_rules(policy, self._sg_id)
         m_get_ns_cidr.assert_called()
         calls = [mock.call(self._sg_id, 'ingress', port_range_min=1,
-                           port_range_max=65535, cidr=subnet_cidr),
+                           port_range_max=65535, cidr=subnet_cidr,
+                           namespace=''),
                  mock.call(self._sg_id, 'egress', port_range_min=1,
-                           port_range_max=65535, cidr=subnet_cidr)]
+                           port_range_max=65535, cidr=subnet_cidr,
+                           namespace='')]
         m_create.assert_has_calls(calls)
 
     def test_knps_on_namespace(self):
