@@ -66,17 +66,13 @@ class TestVIFHandler(test_base.TestCase):
         self._request_vif = self._handler._drv_vif_pool.request_vif
         self._release_vif = self._handler._drv_vif_pool.release_vif
         self._activate_vif = self._handler._drv_vif_pool.activate_vif
-        self._get_pod_state = self._handler._get_pod_state
         self._set_pod_state = self._handler._set_pod_state
-        self._is_host_network = self._handler._is_host_network
         self._is_pending_node = self._handler._is_pending_node
         self._request_additional_vifs = \
             self._multi_vif_drv.request_additional_vifs
 
         self._request_vif.return_value = self._vif
         self._request_additional_vifs.return_value = self._additioan_vifs
-        self._get_pod_state.return_value = self._state
-        self._is_host_network.return_value = False
         self._is_pending_node.return_value = True
         self._get_project.return_value = self._project_id
         self._get_subnets.return_value = self._subnets
@@ -84,9 +80,6 @@ class TestVIFHandler(test_base.TestCase):
         self._set_vifs_driver.return_value = mock.Mock(
             spec=drivers.PodVIFDriver)
 
-        self._handler._drv_for_vif = h_vif.VIFHandler._drv_for_vif.__get__(
-            self._handler)
-
     @mock.patch.object(drivers.MultiVIFDriver, 'get_enabled_drivers')
     @mock.patch.object(drivers.VIFPoolDriver, 'set_vif_driver')
     @mock.patch.object(drivers.VIFPoolDriver, 'get_instance')
@@ -118,18 +111,6 @@ class TestVIFHandler(test_base.TestCase):
         self.assertEqual(vif_pool_driver, handler._drv_vif_pool)
         self.assertEqual(multi_vif_drivers, handler._drv_multi_vif)
 
-    def test_is_host_network(self):
-        self._pod['spec']['hostNetwork'] = True
-        self.assertTrue(h_vif.VIFHandler._is_host_network(self._pod))
-
-    def test_is_not_host_network(self):
-        self.assertFalse(h_vif.VIFHandler._is_host_network(self._pod))
-
-    def test_unset_host_network(self):
-        pod = self._pod.copy()
-        del pod['spec']['hostNetwork']
-        self.assertFalse(h_vif.VIFHandler._is_host_network(pod))
-
     def test_is_pending_node(self):
         self.assertTrue(h_vif.VIFHandler._is_pending_node(self._pod))
 
@@ -145,54 +126,72 @@ class TestVIFHandler(test_base.TestCase):
         self.assertFalse(h_vif.VIFHandler._is_pending_node({'spec': {},
                                                             'status': {}}))
 
-    def test_on_present(self):
+    @mock.patch('kuryr_kubernetes.controller.drivers.utils.is_host_network')
+    @mock.patch('kuryr_kubernetes.controller.drivers.utils.get_pod_state')
+    def test_on_present(self, m_get_pod_state, m_host_network):
+        m_get_pod_state.return_value = self._state
+        m_host_network.return_value = False
         h_vif.VIFHandler.on_present(self._handler, self._pod)
 
-        self._get_pod_state.assert_called_once_with(self._pod)
+        m_get_pod_state.assert_called_once_with(self._pod)
         self._request_vif.assert_not_called()
         self._request_additional_vifs.assert_not_called()
         self._activate_vif.assert_not_called()
         self._set_pod_state.assert_not_called()
 
-    def test_on_present_host_network(self):
-        self._is_host_network.return_value = True
+    @mock.patch('kuryr_kubernetes.controller.drivers.utils.is_host_network')
+    @mock.patch('kuryr_kubernetes.controller.drivers.utils.get_pod_state')
+    def test_on_present_host_network(self, m_get_pod_state, m_host_network):
+        m_get_pod_state.return_value = self._state
+        m_host_network.return_value = True
 
         h_vif.VIFHandler.on_present(self._handler, self._pod)
 
-        self._get_pod_state.assert_not_called()
+        m_get_pod_state.assert_not_called()
         self._request_vif.assert_not_called()
         self._request_additional_vifs.assert_not_called()
         self._activate_vif.assert_not_called()
         self._set_pod_state.assert_not_called()
 
-    def test_on_present_not_pending(self):
+    @mock.patch('kuryr_kubernetes.controller.drivers.utils.is_host_network')
+    @mock.patch('kuryr_kubernetes.controller.drivers.utils.get_pod_state')
+    def test_on_present_not_pending(self, m_get_pod_state, m_host_network):
+        m_get_pod_state.return_value = self._state
+        m_host_network.return_value = False
         self._is_pending_node.return_value = False
 
         h_vif.VIFHandler.on_present(self._handler, self._pod)
 
-        self._get_pod_state.assert_not_called()
+        m_get_pod_state.assert_not_called()
         self._request_vif.assert_not_called()
         self._request_additional_vifs.assert_not_called()
         self._activate_vif.assert_not_called()
         self._set_pod_state.assert_not_called()
 
-    def test_on_present_activate(self):
+    @mock.patch('kuryr_kubernetes.controller.drivers.utils.is_host_network')
+    @mock.patch('kuryr_kubernetes.controller.drivers.utils.get_pod_state')
+    def test_on_present_activate(self, m_get_pod_state, m_host_network):
+        m_get_pod_state.return_value = self._state
+        m_host_network.return_value = False
         self._vif.active = False
 
         h_vif.VIFHandler.on_present(self._handler, self._pod)
 
-        self._get_pod_state.assert_called_once_with(self._pod)
+        m_get_pod_state.assert_called_once_with(self._pod)
         self._activate_vif.assert_called_once_with(self._pod, self._vif)
         self._set_pod_state.assert_called_once_with(self._pod, self._state)
         self._request_vif.assert_not_called()
         self._request_additional_vifs.assert_not_called()
 
-    def test_on_present_create(self):
-        self._get_pod_state.return_value = None
+    @mock.patch('kuryr_kubernetes.controller.drivers.utils.is_host_network')
+    @mock.patch('kuryr_kubernetes.controller.drivers.utils.get_pod_state')
+    def test_on_present_create(self, m_get_pod_state, m_host_network):
+        m_get_pod_state.return_value = None
+        m_host_network.return_value = False
 
         h_vif.VIFHandler.on_present(self._handler, self._pod)
 
-        self._get_pod_state.assert_called_once_with(self._pod)
+        m_get_pod_state.assert_called_once_with(self._pod)
         self._request_vif.assert_called_once_with(
             self._pod, self._project_id, self._subnets, self._security_groups)
         self._request_additional_vifs.assert_called_once_with(
@@ -200,15 +199,19 @@ class TestVIFHandler(test_base.TestCase):
         self._set_pod_state.assert_called_once_with(self._pod, self._state)
         self._activate_vif.assert_not_called()
 
-    def test_on_present_create_with_additional_vifs(self):
-        self._get_pod_state.return_value = None
+    @mock.patch('kuryr_kubernetes.controller.drivers.utils.is_host_network')
+    @mock.patch('kuryr_kubernetes.controller.drivers.utils.get_pod_state')
+    def test_on_present_create_with_additional_vifs(self, m_get_pod_state,
+                                                    m_host_network):
+        m_get_pod_state.return_value = None
+        m_host_network.return_value = False
         additional_vif = os_obj.vif.VIFBase()
         self._state.additional_vifs = {'eth1': additional_vif}
         self._request_additional_vifs.return_value = [additional_vif]
 
         h_vif.VIFHandler.on_present(self._handler, self._pod)
 
-        self._get_pod_state.assert_called_once_with(self._pod)
+        m_get_pod_state.assert_called_once_with(self._pod)
         self._request_vif.assert_called_once_with(
             self._pod, self._project_id, self._subnets, self._security_groups)
         self._request_additional_vifs.assert_called_once_with(
@@ -216,13 +219,16 @@ class TestVIFHandler(test_base.TestCase):
         self._set_pod_state.assert_called_once_with(self._pod, self._state)
         self._activate_vif.assert_not_called()
 
-    def test_on_present_rollback(self):
-        self._get_pod_state.return_value = None
+    @mock.patch('kuryr_kubernetes.controller.drivers.utils.is_host_network')
+    @mock.patch('kuryr_kubernetes.controller.drivers.utils.get_pod_state')
+    def test_on_present_rollback(self, m_get_pod_state, m_host_network):
+        m_get_pod_state.return_value = None
+        m_host_network.return_value = False
         self._set_pod_state.side_effect = k_exc.K8sClientException
 
         h_vif.VIFHandler.on_present(self._handler, self._pod)
 
-        self._get_pod_state.assert_called_once_with(self._pod)
+        m_get_pod_state.assert_called_once_with(self._pod)
         self._request_vif.assert_called_once_with(
             self._pod, self._project_id, self._subnets, self._security_groups)
         self._request_additional_vifs.assert_called_once_with(
@@ -233,18 +239,26 @@ class TestVIFHandler(test_base.TestCase):
                                                   self._security_groups)
         self._activate_vif.assert_not_called()
 
-    def test_on_deleted(self):
+    @mock.patch('kuryr_kubernetes.controller.drivers.utils.is_host_network')
+    @mock.patch('kuryr_kubernetes.controller.drivers.utils.get_pod_state')
+    def test_on_deleted(self, m_get_pod_state, m_host_network):
+        m_get_pod_state.return_value = self._state
+        m_host_network.return_value = False
         h_vif.VIFHandler.on_deleted(self._handler, self._pod)
 
-        self._get_pod_state.assert_called_once_with(self._pod)
+        m_get_pod_state.assert_called_once_with(self._pod)
         self._release_vif.assert_called_once_with(self._pod, self._vif,
                                                   self._project_id,
                                                   self._security_groups)
 
-    def test_on_deleted_with_additional_vifs(self):
+    @mock.patch('kuryr_kubernetes.controller.drivers.utils.is_host_network')
+    @mock.patch('kuryr_kubernetes.controller.drivers.utils.get_pod_state')
+    def test_on_deleted_with_additional_vifs(self, m_get_pod_state,
+                                             m_host_network):
         additional_vif = os_obj.vif.VIFBase()
         self._state.additional_vifs = {'eth1': additional_vif}
-        self._get_pod_state.return_value = self._state
+        m_get_pod_state.return_value = self._state
+        m_host_network.return_value = False
 
         h_vif.VIFHandler.on_deleted(self._handler, self._pod)
 
@@ -255,18 +269,24 @@ class TestVIFHandler(test_base.TestCase):
                                           self._project_id,
                                           self._security_groups)
 
-    def test_on_deleted_host_network(self):
-        self._is_host_network.return_value = True
+    @mock.patch('kuryr_kubernetes.controller.drivers.utils.is_host_network')
+    @mock.patch('kuryr_kubernetes.controller.drivers.utils.get_pod_state')
+    def test_on_deleted_host_network(self, m_get_pod_state, m_host_network):
+        m_get_pod_state.return_value = self._state
+        m_host_network.return_value = True
 
         h_vif.VIFHandler.on_deleted(self._handler, self._pod)
 
-        self._get_pod_state.assert_not_called()
+        m_get_pod_state.assert_not_called()
         self._release_vif.assert_not_called()
 
-    def test_on_deleted_no_annotation(self):
-        self._get_pod_state.return_value = None
+    @mock.patch('kuryr_kubernetes.controller.drivers.utils.is_host_network')
+    @mock.patch('kuryr_kubernetes.controller.drivers.utils.get_pod_state')
+    def test_on_deleted_no_annotation(self, m_get_pod_state, m_host_network):
+        m_get_pod_state.return_value = None
+        m_host_network.return_value = False
 
         h_vif.VIFHandler.on_deleted(self._handler, self._pod)
 
-        self._get_pod_state.assert_called_once_with(self._pod)
+        m_get_pod_state.assert_called_once_with(self._pod)
         self._release_vif.assert_not_called()
