@@ -16,6 +16,7 @@ import time
 
 import requests
 
+from neutronclient.common import exceptions as n_exc
 from os_vif import objects
 from oslo_cache import core as cache
 from oslo_config import cfg
@@ -23,6 +24,7 @@ from oslo_log import log
 from oslo_serialization import jsonutils
 
 from kuryr_kubernetes import clients
+from kuryr_kubernetes import exceptions
 from kuryr_kubernetes.objects import vif
 from kuryr_kubernetes import os_vif_util
 
@@ -160,6 +162,17 @@ def get_subnet(subnet_id):
     return network
 
 
+@MEMOIZE
+def get_subnet_cidr(subnet_id):
+    neutron = clients.get_neutron_client()
+    try:
+        subnet_obj = neutron.show_subnet(subnet_id)
+    except n_exc.NeutronClientException:
+        LOG.exception("Subnet %s CIDR not found!", subnet_id)
+        raise
+    return subnet_obj.get('subnet')['cidr']
+
+
 def extract_pod_annotation(annotation):
     obj = objects.base.VersionedObject.obj_from_primitive(annotation)
     # FIXME(dulek): This is code to maintain compatibility with Queens. We can
@@ -186,3 +199,14 @@ def is_available(resource, resource_quota, neutron_func):
         LOG.error("Quota exceeded for resource: %s", resource)
         return False
     return True
+
+
+def has_kuryr_crd(crd_url):
+    k8s = clients.get_kubernetes_client()
+    try:
+        k8s.get(crd_url, json=False, headers={'Connection': 'close'})
+    except exceptions.K8sClientException:
+        LOG.exception("Kubernetes Client Exception fetching"
+                      " CRD. %s" % exceptions.K8sClientException)
+        return False
+    return True
